<?php
namespace Ak\App\Middleware;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

/**
 * Omogućava segregaciju middlewareova po metodi requesta.
 *
 * @package Ak\App\Middleware
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/app
 */
final class RequestMethodMiddleware implements MiddlewareInterface
{
    /** @var string[] */
    private $methods;
    /** @var MiddlewareInterface */
    private $middleware;

    /**
     * Konstruktor.
     *
     * @param string|array $methodOrMethods
     * @param MiddlewareInterface $middleware
     */
    public function __construct($methodOrMethods, MiddlewareInterface $middleware)
    {
        $this->methods = is_array($methodOrMethods) ? $methodOrMethods : [$methodOrMethods];
        $this->middleware = $middleware;
    }

    /**
     * Ako metoda datog requesta odgovara zadanoj metodi izvršava zadani
     * middleware, inače prosljeđuje kontrolu sljedećem u nizu.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        if (in_array($request->getMethod(), $this->methods)) {
            return $this->middleware->process($request, $handler);
        }

        return $handler->handle($request);
    }
}