<?php
namespace AkTest\App\Middelware;

use Ak\App\Middleware\RequestMethodMiddleware;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Diactoros\ServerRequest;

/**
 * Class RequestMethodMiddlewareTest
 *
 * @package AkTest\App\Middelware
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/app
 */
final class RequestMethodMiddlewareTest extends TestCase
{
    public function testWillMatchRequestMethod()
    {
        $request = new ServerRequest([], [], null, 'POST');
        $response = $this->createMock(ResponseInterface::class);

        /** @var RequestHandlerInterface|MockObject $handler */
        $handler = $this->createMock(RequestHandlerInterface::class);
        $handler->expects($this->never())
            ->method('handle');

        /** @var MiddlewareInterface|MockObject $inner */
        $inner = $this->createMock(MiddlewareInterface::class);
        $inner->expects($this->once())
            ->method('process')
            ->with($request, $handler)
            ->willReturn($response);

        $middleware = new RequestMethodMiddleware('POST', $inner);
        $this->assertSame($response, $middleware->process($request, $handler));
    }

    public function testWillPassNonMatchedRequestMethod()
    {
        $request = new ServerRequest([], [], null, 'POST');
        $response = $this->createMock(ResponseInterface::class);

        /** @var RequestHandlerInterface|MockObject $handler */
        $handler = $this->createMock(RequestHandlerInterface::class);
        $handler->expects($this->once())
            ->method('handle')
            ->with($request)
            ->willReturn($response);

        /** @var MiddlewareInterface|MockObject $inner */
        $inner = $this->createMock(MiddlewareInterface::class);
        $inner->expects($this->never())
            ->method('process');

        $middleware = new RequestMethodMiddleware('GET', $inner);
        $this->assertSame($response, $middleware->process($request, $handler));
    }
}