<?php
namespace AkTest\App\View\Helper;

use Ak\App\ConfigProvider;
use Ak\Test\AbstractTestCase;
use Blast\BaseUrl\BasePathHelper;
use Blast\BaseUrl\BasePathViewHelper;
use Blast\BaseUrl\BaseUrlMiddleware;
use PHPUnit\Framework\MockObject\MockObject;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\UriInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Diactoros\ServerRequestFactory;
use Zend\Diactoros\Uri;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\View\Helper\BasePath;

final class BasePathTest extends AbstractTestCase
{
    public $configProviders = [
        ConfigProvider::class,
    ];

    public function testBasePathCanBeCreated()
    {
        $container = $this->getContainer();
        /** @var PluginManagerInterface $helpers */
        $helpers = $container->get('ViewHelperManager');

        $this->assertContainerHasService($helpers, [
            'base_path',
            'basePath',
            BasePath::class,
            BasePathHelper::class,
        ], BasePathViewHelper::class, true);
    }

    public function testCanPrependBaseUrlToUrl()
    {
        /** @var UriInterface|MockObject $uri */
        $uri = new Uri('https://localhost/subdir/index.php');

        $request = ServerRequestFactory::fromGlobals([
            'SCRIPT_FILENAME' => '/var/www/subdir/index.php',
            'SCRIPT_NAME' => '/subdir/index.php',
            'PHP_SELF' => '/subdir/index.php',
            'ORIG_SCRIPT_NAME' => '',
        ]);
        $request = $request->withUri($uri);

        $response = $this->createMock(ResponseInterface::class);

        /** @var RequestHandlerInterface|MockObject $handler */
        $handler = $this->createMock(RequestHandlerInterface::class);
        $handler->expects($this->once())
            ->method('handle')
            ->willReturn($response);

        $container = $this->getContainer();

        /** @var BaseUrlMiddleware $baseUrl */
        $baseUrl = $container->get(BaseUrlMiddleware::class);
        /** @noinspection PhpParamsInspection */
        $baseUrl->process($request, $handler);

        /** @var PluginManagerInterface $helpers */
        $helpers = $container->get('ViewHelperManager');
        /** @var callable $helper */
        $helper = $helpers->get('base_path');

        $this->assertEquals('/subdir/some-path/image.jpg', $helper('/some-path/image.jpg'));
    }
}